@@ -4,6 +4,12 @@
 """
 Pyramid authn policy that ties together multiple backends.
 """
+import sys
+
+from zope.interface import implementer
+
+from pyramid.interfaces import IAuthenticationPolicy, PHASE2_CONFIG
+from pyramid.security import Everyone, Authenticated
 
 __ver_major__ = 0
 __ver_minor__ = 4
@@ -13,15 +19,6 @@ __ver_tuple__ = (__ver_major__, __ver_minor__, __ver_patch__, __ver_sub__)
 __version__ = "%d.%d.%d%s" % __ver_tuple__
 
 
-import sys
-
-from zope.interface import implementer
-
-from pyramid.interfaces import IAuthenticationPolicy, PHASE2_CONFIG
-from pyramid.security import Everyone, Authenticated
-from pyramid.authorization import ACLAuthorizationPolicy
-
-
 if sys.version_info > (3,):  # pragma: nocover
     basestring = str
 
@@ -42,6 +39,7 @@ class MultiAuthPolicySelected(object):
     """
     def __init__(self, policy, request):
         self.policy = policy
+        self.policy_name = getattr(policy, "_pyramid_multiauth_name", None)
         self.request = request
 
 
@@ -78,7 +76,8 @@ class MultiAuthenticationPolicy(object):
         for policy in self._policies:
             userid = policy.authenticated_userid(request)
             if userid is not None:
-                request.registry.notify(MultiAuthPolicySelected(policy, request))
+                request.registry.notify(MultiAuthPolicySelected(policy,
+                                                                request))
 
                 if self._callback is None:
                     break
@@ -149,6 +148,16 @@ class MultiAuthenticationPolicy(object):
             headers.extend(policy.forget(request))
         return headers
 
+    def get_policies(self):
+        """Get the list of contained authentication policies, as tuple of
+        name and instances.
+
+        This may be useful to instrospect the configured policies, and their
+        respective name defined in configuration.
+        """
+        return [(getattr(policy, "_pyramid_multiauth_name", None), policy)
+                for policy in self._policies]
+
     def get_policy(self, name_or_class):
         """Get one of the contained authentication policies, by name or class.
 
@@ -207,14 +216,23 @@ def includeme(config):
 
     As a side-effect, the configuration will also get the additional views
     that pyramid_browserid sets up by default.
+
+    The *group finder function* and the *authorization policy* are also read
+    from configuration if specified:
+
+        multiauth.authorization_policy = mypyramidapp.acl.Custom
+        multiauth.groupfinder  = mypyramidapp.acl.groupfinder
     """
     # Grab the pyramid-wide settings, to look for any auth config.
     settings = config.get_settings()
     # Hook up a default AuthorizationPolicy.
-    # ACLAuthorizationPolicy is usually what you want.
+    # Get the authorization policy from config if present.
+    # Default ACLAuthorizationPolicy is usually what you want.
+    authz_class = settings.get("multiauth.authorization_policy",
+                               "pyramid.authorization.ACLAuthorizationPolicy")
+    authz_policy = config.maybe_dotted(authz_class)()
     # If the app configures one explicitly then this will get overridden.
     # In autocommit mode this needs to be done before setting the authn policy.
-    authz_policy = ACLAuthorizationPolicy()
     config.set_authorization_policy(authz_policy)
     # Get the groupfinder from config if present.
     groupfinder = settings.get("multiauth.groupfinder", None)
@@ -251,7 +269,8 @@ def includeme(config):
     # commit time, and will return the policies in reverse order.
     # Register a special action to pull them into our list of policies.
     policies = []
-    def grab_policies():  # NOQA
+
+    def grab_policies():
         for factory, name, kwds in policy_factories:
             policy = factory(**kwds)
             if policy:
@@ -260,6 +279,7 @@ def includeme(config):
                     # Remember, they're being processed in reverse order.
                     # So each new policy needs to go at the front.
                     policies.insert(0, policy)
+
     config.action(None, grab_policies, order=PHASE2_CONFIG)
     authn_policy = MultiAuthenticationPolicy(policies, groupfinder)
     config.set_authentication_policy(authn_policy)
@@ -296,13 +316,15 @@ def policy_factory_from_module(config, module):
         # If it's not setting the authn policy, keep looking.
         if discriminator is not IAuthenticationPolicy:
             continue
+
         # Otherwise, wrap it up so we can extract the registered object.
-        def grab_policy(register=callable):  # NOQA
+        def grab_policy(register=callable):
             old_policy = config.registry.queryUtility(IAuthenticationPolicy)
             register()
             new_policy = config.registry.queryUtility(IAuthenticationPolicy)
             config.registry.registerUtility(old_policy, IAuthenticationPolicy)
             return new_policy
+
         return grab_policy
     # Or it might not have done *anything*.
     # So return a null policy factory.
