@@ -3,21 +3,22 @@
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import sys
-if sys.version_info < (2, 7):
-    import unittest2 as unittest  # pragma: nocover
-else:
-    import unittest  # pragma: nocover  # NOQA
-
 from zope.interface import implementer
 
+import pyramid.authorization
 import pyramid.testing
 from pyramid.testing import DummyRequest
 from pyramid.security import Everyone, Authenticated
 from pyramid.exceptions import Forbidden
-from pyramid.interfaces import IAuthenticationPolicy
+from pyramid.interfaces import IAuthenticationPolicy, IAuthorizationPolicy
 
 from pyramid_multiauth import MultiAuthenticationPolicy
 
+if sys.version_info < (2, 7):
+    import unittest2 as unittest  # pragma: nocover
+else:
+    import unittest  # pragma: nocover
+
 
 #  Here begins various helper classes and functions for the tests.
 
@@ -104,11 +105,22 @@ class TestAuthnPolicyUnauthOnly(BaseAuthnPolicy):
         return [Everyone]
 
 
+@implementer(IAuthorizationPolicy)
+class TestAuthzPolicyCustom(object):
+    def permits(self, context, principals, permission):
+        return True
+
+    def principals_allowed_by_permission(self, context, permission):
+        raise NotImplementedError()  # pragma: nocover
+
+
 def testincludeme1(config):
     """Config include that sets up a TestAuthnPolicy1 and a forbidden view."""
     config.set_authentication_policy(TestAuthnPolicy1())
-    def forbidden_view(request):  # NOQA
+
+    def forbidden_view(request):
         return "FORBIDDEN ONE"
+
     config.add_view(forbidden_view,
                     renderer="json",
                     context="pyramid.exceptions.Forbidden")
@@ -169,6 +181,8 @@ class MultiAuthPolicyTests(unittest.TestCase):
 
         policies = [TestAuthnPolicy2(), TestAuthnPolicy3()]
         policy = MultiAuthenticationPolicy(policies)
+        # Simulate loading from config:
+        policies[0]._pyramid_multiauth_name = "name"
 
         with testConfig() as config:
             request = DummyRequest()
@@ -176,15 +190,15 @@ class MultiAuthPolicyTests(unittest.TestCase):
             selected_policy = []
 
             def track_policy(event):
-                selected_policy.append((event.policy, event.request))
+                selected_policy.append(event)
 
             config.add_subscriber(track_policy, MultiAuthPolicySelected)
 
-
             self.assertEquals(policy.authenticated_userid(request), "test2")
 
-            self.assertEquals(selected_policy[0][0], policies[0])
-            self.assertEquals(selected_policy[0][1], request)
+            self.assertEquals(selected_policy[0].policy, policies[0])
+            self.assertEquals(selected_policy[0].policy_name, "name")
+            self.assertEquals(selected_policy[0].request, request)
             self.assertEquals(len(selected_policy), 1)
 
     def test_stacking_of_unauthenticated_userid(self):
@@ -262,23 +276,40 @@ class MultiAuthPolicyTests(unittest.TestCase):
         policy = MultiAuthenticationPolicy(policies)
         request = DummyRequest()
         self.assertEquals(policy.remember(request, "ha"),
-                         [("X-Remember", "ha"), ("X-Remember-2", "ha")])
+                          [("X-Remember", "ha"), ("X-Remember-2", "ha")])
         self.assertEquals(policy.forget(request),
-                         [("X-Forget", "foo"), ("X-Forget", "bar")])
+                          [("X-Forget", "foo"), ("X-Forget", "bar")])
         policies.reverse()
         self.assertEquals(policy.remember(request, "ha"),
-                         [("X-Remember-2", "ha"), ("X-Remember", "ha")])
+                          [("X-Remember-2", "ha"), ("X-Remember", "ha")])
         self.assertEquals(policy.forget(request),
-                         [("X-Forget", "bar"), ("X-Forget", "foo")])
+                          [("X-Forget", "bar"), ("X-Forget", "foo")])
+
+    def test_includeme_uses_acl_authorization_by_default(self):
+        self.config.include("pyramid_multiauth")
+        self.config.commit()
+        policy = self.config.registry.getUtility(IAuthorizationPolicy)
+        expected = pyramid.authorization.ACLAuthorizationPolicy
+        self.assertTrue(isinstance(policy, expected))
+
+    def test_includeme_reads_authorization_from_settings(self):
+        self.config.add_settings({
+            "multiauth.authorization_policy": "pyramid_multiauth.tests."
+            "TestAuthzPolicyCustom"
+        })
+        self.config.include("pyramid_multiauth")
+        self.config.commit()
+        policy = self.config.registry.getUtility(IAuthorizationPolicy)
+        self.assertTrue(isinstance(policy, TestAuthzPolicyCustom))
 
     def test_includeme_by_module(self):
         self.config.add_settings({
-          "multiauth.groupfinder": "pyramid_multiauth.tests.testgroupfinder",
-          "multiauth.policies": "pyramid_multiauth.tests.testincludeme1 "\
-                                "pyramid_multiauth.tests.testincludeme2 "\
-                                "pyramid_multiauth.tests.testincludemenull "\
-                                "pyramid_multiauth.tests.testincludeme3 "\
-        })  # NOQA
+            "multiauth.groupfinder": "pyramid_multiauth.tests.testgroupfinder",
+            "multiauth.policies": "pyramid_multiauth.tests.testincludeme1 "
+                                  "pyramid_multiauth.tests.testincludeme2 "
+                                  "pyramid_multiauth.tests.testincludemenull "
+                                  "pyramid_multiauth.tests.testincludeme3 "
+        })
         self.config.include("pyramid_multiauth")
         self.config.commit()
         policy = self.config.registry.getUtility(IAuthenticationPolicy)
@@ -293,24 +324,26 @@ class MultiAuthPolicyTests(unittest.TestCase):
         self.config.add_view(raiseforbidden, route_name="index")
         app = self.config.make_wsgi_app()
         environ = {"PATH_INFO": "/", "REQUEST_METHOD": "GET"}
-        def start_response(*args):  # NOQA
+
+        def start_response(*args):
             pass
+
         result = b"".join(app(environ, start_response))
         self.assertEquals(result, b'"FORBIDDEN ONE"')
 
     def test_includeme_by_callable(self):
         self.config.add_settings({
-          "multiauth.groupfinder":
-                  "pyramid_multiauth.tests.testgroupfinder",
-          "multiauth.policies":
-                  "pyramid_multiauth.tests.testincludeme1 policy1 policy2",
-          "multiauth.policy.policy1.use":
-                  "pyramid_multiauth.tests.TestAuthnPolicy2",
-          "multiauth.policy.policy1.foo":
-                  "bar",
-          "multiauth.policy.policy2.use":
-                   "pyramid_multiauth.tests.TestAuthnPolicy3"
-        })  # NOQA
+            "multiauth.groupfinder":
+                "pyramid_multiauth.tests.testgroupfinder",
+            "multiauth.policies":
+                "pyramid_multiauth.tests.testincludeme1 policy1 policy2",
+            "multiauth.policy.policy1.use":
+                "pyramid_multiauth.tests.TestAuthnPolicy2",
+            "multiauth.policy.policy1.foo":
+                "bar",
+            "multiauth.policy.policy2.use":
+                "pyramid_multiauth.tests.TestAuthnPolicy3"
+        })
         self.config.include("pyramid_multiauth")
         self.config.commit()
         policy = self.config.registry.getUtility(IAuthenticationPolicy)
@@ -326,34 +359,36 @@ class MultiAuthPolicyTests(unittest.TestCase):
         self.config.add_view(raiseforbidden, route_name="index")
         app = self.config.make_wsgi_app()
         environ = {"PATH_INFO": "/", "REQUEST_METHOD": "GET"}
-        def start_response(*args):  # NOQA
+
+        def start_response(*args):
             pass
+
         result = b"".join(app(environ, start_response))
         self.assertEquals(result, b'"FORBIDDEN ONE"')
 
     def test_includeme_with_unconfigured_policy(self):
         self.config.add_settings({
-          "multiauth.groupfinder":
-                  "pyramid_multiauth.tests.testgroupfinder",
-          "multiauth.policies":
-                  "pyramid_multiauth.tests.testincludeme1 policy1 policy2",
-          "multiauth.policy.policy1.use":
-                  "pyramid_multiauth.tests.TestAuthnPolicy2",
-          "multiauth.policy.policy1.foo":
-                  "bar",
+            "multiauth.groupfinder":
+                "pyramid_multiauth.tests.testgroupfinder",
+            "multiauth.policies":
+                "pyramid_multiauth.tests.testincludeme1 policy1 policy2",
+            "multiauth.policy.policy1.use":
+                "pyramid_multiauth.tests.TestAuthnPolicy2",
+            "multiauth.policy.policy1.foo":
+                "bar",
         })
         self.assertRaises(ValueError, self.config.include, "pyramid_multiauth")
 
     def test_get_policy(self):
         self.config.add_settings({
-          "multiauth.policies":
-                  "pyramid_multiauth.tests.testincludeme1 policy1 policy2",
-          "multiauth.policy.policy1.use":
-                  "pyramid_multiauth.tests.TestAuthnPolicy2",
-          "multiauth.policy.policy1.foo":
-                  "bar",
-          "multiauth.policy.policy2.use":
-                   "pyramid_multiauth.tests.TestAuthnPolicy3"
+            "multiauth.policies":
+                "pyramid_multiauth.tests.testincludeme1 policy1 policy2",
+            "multiauth.policy.policy1.use":
+                "pyramid_multiauth.tests.TestAuthnPolicy2",
+            "multiauth.policy.policy1.foo":
+                "bar",
+            "multiauth.policy.policy2.use":
+                "pyramid_multiauth.tests.TestAuthnPolicy3"
         })
         self.config.include("pyramid_multiauth")
         self.config.commit()
@@ -372,3 +407,25 @@ class MultiAuthPolicyTests(unittest.TestCase):
         self.assertTrue(isinstance(policy.get_policy(TestAuthnPolicy3),
                                    TestAuthnPolicy3))
         self.assertEquals(policy.get_policy(MultiAuthPolicyTests), None)
+
+    def test_get_policies(self):
+        self.config.add_settings({
+            "multiauth.policies":
+                "pyramid_multiauth.tests.testincludeme1 policy1 policy2",
+            "multiauth.policy.policy1.use":
+                "pyramid_multiauth.tests.TestAuthnPolicy2",
+            "multiauth.policy.policy2.use":
+                "pyramid_multiauth.tests.TestAuthnPolicy3"
+        })
+        self.config.include("pyramid_multiauth")
+        self.config.commit()
+        policy = self.config.registry.getUtility(IAuthenticationPolicy)
+        policies = policy.get_policies()
+        expected_result = [
+            ("pyramid_multiauth.tests.testincludeme1", TestAuthnPolicy1),
+            ("policy1", TestAuthnPolicy2),
+            ("policy2", TestAuthnPolicy3),
+        ]
+        for (obtained, expected) in zip(policies, expected_result):
+            self.assertEquals(obtained[0], expected[0])
+            self.assertTrue(isinstance(obtained[1], expected[1]))
